#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cnfg_defs.h"
#include "funct_codes.h"

extern char wbuf[];
extern int nwbuf;
extern char rbuf[];
extern int nrbuf;

CNFG_ptr load_cnfg(char *fn)
{
  int i;
  char line[CNFG_MAXLINE],key_wrd[24];
  static char *fmt[] = {"%02X-","%02X-","%02X-","%02X-","%02X-","%02X"};
  CNFG_ptr cp;
  FILE *fp;
  if((fp=fopen(fn,"r"))==0){
    return((CNFG_ptr)(-1));
  }
  cp=(CNFG_ptr)malloc(sizeof(CNFG_t));
  while(fscanf(fp,"%s",key_wrd)>0){
    if(key_wrd[0]!='#'&&(strncmp(key_wrd,"//",2)!=0)){
      if(strcmp(key_wrd,"Device:")==0){
        for(i=0;i<6;i++) fscanf(fp,fmt[i],cp->mac.device+i);
      }
      else if(strcmp(key_wrd,"Mcast1:")==0){
        for(i=0;i<6;i++) fscanf(fp,fmt[i],cp->mac.mcast1+i);
      }
      else if(strcmp(key_wrd,"Mcast2:")==0){
        for(i=0;i<6;i++) fscanf(fp,fmt[i],cp->mac.mcast2+i);
      }
      else if(strcmp(key_wrd,"Mcast3:")==0){
        for(i=0;i<6;i++) fscanf(fp,fmt[i],cp->mac.mcast3+i);
      }
      else if(strcmp(key_wrd,"Dflt_Srv:")==0){
        for(i=0;i<6;i++) fscanf(fp,fmt[i],cp->mac.dflt_srv+i);
      }
      else if(strcmp(key_wrd,"Ether:")==0){
        fscanf(fp,"%hX",&cp->ether);
      }
      else if(strcmp(key_wrd,"Ext_FIFO:")==0){
        fscanf(fp,"%hX",&cp->ext_fifo);
      }
      else if(strcmp(key_wrd,"Reset:")==0){
        fscanf(fp,"%hX",&cp->reset);
      }
      else if(strcmp(key_wrd,"VME:")==0){
        fscanf(fp,"%X",&cp->vme);
      }
      else if(strcmp(key_wrd,"VME_BTO:")==0){
        fscanf(fp,"%hX",&cp->vme_bto);
      }
      else if(strcmp(key_wrd,"VME_BGTO:")==0){
        fscanf(fp,"%hX",&cp->vme_bgto);
      }
      else {
        printf("\nUnrecognized Keyword: %s\n",key_wrd);
        fgets(line,CNFG_MAXLINE,fp);
      }
    }
    else {
      fgets(line,CNFG_MAXLINE,fp);
    }
  }
  fclose(fp);
  return(cp);
}
int write_cnfg(CNFG_ptr cp, char *fn)
{
  int i;
  FILE *fp;
  if((fp=fopen(fn,"w"))==0){
    printf("Error could not open %s\n");
    return(-1);
  }
  fprintf(fp,"Device:   ");
  for(i=0;i<5;i++){
    fprintf(fp,"%02X-",cp->mac.device[i]);
  }
  fprintf(fp,"%02X\n",cp->mac.device[5]);
  fprintf(fp,"Mcast1:   ");
  for(i=0;i<5;i++){
    fprintf(fp,"%02X-",cp->mac.mcast1[i]);
  }
  fprintf(fp,"%02X\n",cp->mac.mcast1[5]);
  fprintf(fp,"Mcast2:   ");
  for(i=0;i<5;i++){
    fprintf(fp,"%02X-",cp->mac.mcast2[i]);
  }
  fprintf(fp,"%02X\n",cp->mac.mcast2[5]);
  fprintf(fp,"Mcast3:   ");
  for(i=0;i<5;i++){
    fprintf(fp,"%02X-",cp->mac.mcast3[i]);
  }
  fprintf(fp,"%02X\n",cp->mac.mcast3[5]);
  fprintf(fp,"Dflt_srv: ");
  for(i=0;i<5;i++){
    fprintf(fp,"%02X-",cp->mac.dflt_srv[i]);
  }
  fprintf(fp,"%02X\n",cp->mac.dflt_srv[5]);

  fprintf(fp,"Ether:        %04hX\n",cp->ether);
  fprintf(fp,"Ext_fifo:     %04hX\n",cp->ext_fifo);
  fprintf(fp,"Reset:        %04hX\n",cp->reset);
  fprintf(fp,"VME:      %08X\n",cp->vme);
  fprintf(fp,"VME_BTO:      %04hX\n",cp->vme_bto);
  fprintf(fp,"VME_BGTO:     %04hX\n",cp->vme_bgto);
  fclose(fp);
  return(0);
}

int set_cnfg_dflt(int dflt)
{
  int n,l,lcnt;
  wbuf[0]=0x00;
  wbuf[1]=Set_Cnfg_Dflt;
  wbuf[2]=0x00;
  wbuf[3]=(dflt&0x1f);
  nwbuf=4;
  n=eth_write();
  for(l=0;l<8000;l++)lcnt++;
  return(n);
}

int read_cnfg_dflt()
{
  int n,type;
  wbuf[0]=0x20;
  wbuf[1]=Read_Cnfg_Dflt;
  nwbuf=2;
  n=eth_write();
  n=eth_read();
  type = rbuf[15]&0xff;
  if(type==0xFF){
    return(-(((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF)));
  }
  return(rbuf[23]&0xFF);
}

int save_cnfg_num(int cnum)
{
  int n,l,lcnt;
  wbuf[0]=0x00;
  wbuf[1]=Save_Cnfg_Num;
  wbuf[2]=0x00;
  wbuf[3]=(cnum&0x1f);  // cnum must be <= 20
  nwbuf=4;
  n=eth_write();
  for(l=0;l<8000;l++)lcnt++;
  return(n);
}

int rstr_cnfg_num(int cnum)
{
  int n,l,lcnt;
  wbuf[0]=0x00;
  wbuf[1]=Rstr_Cnfg_Num;
  wbuf[2]=0x00;
  wbuf[3]=(cnum&0x1f);  // cnum must be <= 20
  nwbuf=4;
  n=eth_write();
  for(l=0;l<8000;l++)lcnt++;
  return(n);
}

int read_cnfg_num_dir(int cnum, char **dpp)
{
  int n,type,numwrds;
  wbuf[0]=0x20;
  wbuf[1]=Read_Cnfg_Num_Dir;
  wbuf[2]=0x00;
  wbuf[3]=(cnum&0x1f);  // cnum must be <= 20
  nwbuf=4;
  n=eth_write();
  n=eth_read();
  type = rbuf[15]&0xff;
  if(type==0xFF){
    return(-(((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF)));
  }
  numwrds = ((rbuf[20]&0xff)<<8)|(rbuf[21]&0xff);
  *dpp = &rbuf[22];
  return(numwrds);
}

CNFG_ptr read_cnfg_num_dcd(int cnum)
{
  int n,type;
  CNFG_ptr cp;
  wbuf[0]=0x20;
  wbuf[1]=Read_Cnfg_Num_Dcd;
  wbuf[2]=0x00;
  wbuf[3]=(cnum&0x1f);  // cnum must be <= 20
  nwbuf=4;
  n=eth_write();
  n=eth_read();
  type = rbuf[15]&0xff;
  if(type==0xFF){
    return((CNFG_ptr)(-(((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF))));
  }
  cp=(CNFG_ptr)malloc(sizeof(CNFG_t));
  cp->ether = ((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF);
  cp->ext_fifo = ((rbuf[24]&0xFF)<<8)|(rbuf[25]&0xFF);
  cp->reset = ((rbuf[26]&0xFF)<<8)|(rbuf[27]&0xFF);
  cp->vme = ((rbuf[28]&0xFF)<<24)|((rbuf[29]&0xFF)<<16)|((rbuf[30]&0xFF)<<8)|(rbuf[31]&0xFF);
  cp->vme_bto = ((rbuf[32]&0xFF)<<8)|(rbuf[33]&0xFF);
  cp->vme_bgto = ((rbuf[34]&0xFF)<<8)|(rbuf[35]&0xFF);
  return(cp);
}

CNFG_ptr read_crs()
{
  int n,type;
  CNFG_ptr cp;
  wbuf[0]=0x20;
  wbuf[1]=Read_CRs;
  nwbuf=2;
  n=eth_write();
  n=eth_read();
  type = rbuf[15]&0xff;
  if(type==0xFF){
    return((CNFG_ptr)(-(((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF))));
  }
  cp=(CNFG_ptr)malloc(sizeof(CNFG_t));
  cp->ether = ((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF);
  cp->ext_fifo = ((rbuf[24]&0xFF)<<8)|(rbuf[25]&0xFF);
  cp->reset = ((rbuf[26]&0xFF)<<8)|(rbuf[27]&0xFF);
  cp->vme = ((rbuf[28]&0xFF)<<24)|((rbuf[29]&0xFF)<<16)|((rbuf[30]&0xFF)<<8)|(rbuf[31]&0xFF);
  cp->vme_bto = ((rbuf[32]&0xFF)<<8)|(rbuf[33]&0xFF);
  cp->vme_bgto = ((rbuf[34]&0xFF)<<8)|(rbuf[35]&0xFF);
  return(cp);
}

int set_macs(enum MAC_ID mcid, CNFG_ptr cp)
{
  int i,n,l,lcnt,err;
  err=0;
  wbuf[0]=0x00;
  wbuf[1]=Set_MACs;
  wbuf[2]=0x00;
  wbuf[3]=mcid;
  nwbuf=4;
  switch(mcid){
    case DEVICE:
      for(i=0;i<6;i++)wbuf[4+i]=cp->mac.device[i];
      nwbuf+=6;
      break;
    case MCAST1:
      for(i=0;i<6;i++)wbuf[4+i]=cp->mac.mcast1[i];
      nwbuf+=6;
      break;
    case MCAST2:
      for(i=0;i<6;i++)wbuf[4+i]=cp->mac.mcast2[i];
      nwbuf+=6;
      break;
    case MCAST3:
      for(i=0;i<6;i++)wbuf[4+i]=cp->mac.mcast3[i];
      nwbuf+=6;
      break;
    case DFLT_SRV:
      for(i=0;i<6;i++)wbuf[4+i]=cp->mac.dflt_srv[i];
      nwbuf+=6;
      break;
    case ALL_MACS:
      for(i=0;i<6;i++)wbuf[4+i]=cp->mac.device[i];
      nwbuf+=6;
      for(i=0;i<6;i++)wbuf[10+i]=cp->mac.mcast1[i];
      nwbuf+=6;
      for(i=0;i<6;i++)wbuf[16+i]=cp->mac.mcast2[i];
      nwbuf+=6;
      for(i=0;i<6;i++)wbuf[22+i]=cp->mac.mcast3[i];
      nwbuf+=6;
      for(i=0;i<6;i++)wbuf[28+i]=cp->mac.dflt_srv[i];
      nwbuf+=6;
      break;
    default:
      printf("illeagal MAC ID\n");
      err=-1;
      break;
  }
  if(err!=0){
    return(err);
  }
  n=eth_write();
  for(l=0;l<8000;l++)lcnt++;
  return(n);
}

int read_macs_dir(char **dpp)
{
  int n,type,numwrds;
  wbuf[0]=0x20;
  wbuf[1]=Read_MACs_Dir;
  nwbuf=2;
  n=eth_write();
  n=eth_read();
  type = rbuf[15]&0xff;
  if(type==0xFF){
    return(-(((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF)));
  }
  numwrds = ((rbuf[20]&0xff)<<8)|(rbuf[21]&0xff);
  *dpp = &rbuf[22];
  return(numwrds);
}

CNFG_ptr read_macs_dcd()
{
  int i,n,type;
  CNFG_ptr cp;
  wbuf[0]=0x20;
  wbuf[1]=Read_MACs_Dcd;
  nwbuf=2;
  n=eth_write();
  n=eth_read();
  type = rbuf[15]&0xff;
  if(type==0xFF){
    return((CNFG_ptr)(-(((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF))));
  }
  cp=(CNFG_ptr)malloc(sizeof(CNFG_t));
  for(i=0;i<6;i++)cp->mac.device[i] = rbuf[22+i];
  for(i=0;i<6;i++)cp->mac.mcast1[i] = rbuf[28+i];
  for(i=0;i<6;i++)cp->mac.mcast2[i] = rbuf[34+i];
  for(i=0;i<6;i++)cp->mac.mcast3[i] = rbuf[40+i];
  for(i=0;i<6;i++)cp->mac.dflt_srv[i] = rbuf[46+i];
  return(cp);
}


//wrt_crs the way I intend to do it
int wrt_crs(enum CR_ID crid, CNFG_ptr cp)
{
  int i,n,l,lcnt,err;
  err=0;
  wbuf[0]=0x00;
  wbuf[1]=Wrt_CR_ID;
  wbuf[2]=0x00;
  wbuf[3]=crid;
  nwbuf=4;
  switch(crid){
    case ETHER:
      wbuf[4]=((cp->ether)>>8)&0xFF;
      wbuf[5]=(cp->ether)&0xFF;
      nwbuf+=2;
      break;
    case EXTFIFO:
      wbuf[4]=((cp->ext_fifo)>>8)&0xFF;
      wbuf[5]=(cp->ext_fifo)&0xFF;
      nwbuf+=2;
      break;
    case RESET:
      wbuf[4]=((cp->reset)>>8)&0xFF;
      wbuf[5]=(cp->reset)&0xFF;
      nwbuf+=2;
      break;
    case VME:
      wbuf[4]=((cp->vme)>>24)&0xFF;
      wbuf[5]=((cp->vme)>>16)&0xFF;
      wbuf[6]=((cp->vme)>>8)&0xFF;
      wbuf[7]=(cp->vme)&0xFF;
      nwbuf+=4;
      break;
    case BTO:
      wbuf[4]=((cp->vme_bto)>>8)&0xFF;
      wbuf[5]=(cp->vme_bto)&0xFF;
      nwbuf+=2;
      break;
    case BGTO:
      wbuf[4]=((cp->vme_bgto)>>8)&0xFF;
      wbuf[5]=(cp->vme_bgto)&0xFF;
      nwbuf+=2;
      break;
    case ALL_CRS:
      wbuf[4]=((cp->ether)>>8)&0xFF;
      wbuf[5]=(cp->ether)&0xFF;
      wbuf[6]=((cp->ext_fifo)>>8)&0xFF;
      wbuf[7]=(cp->ext_fifo)&0xFF;
      wbuf[8]=((cp->reset)>>8)&0xFF;
      wbuf[9]=(cp->reset)&0xFF;
      wbuf[10]=((cp->vme)>>24)&0xFF;
      wbuf[11]=((cp->vme)>>16)&0xFF;
      wbuf[12]=((cp->vme)>>8)&0xFF;
      wbuf[13]=(cp->vme)&0xFF;
      wbuf[14]=((cp->vme_bto)>>8)&0xFF;
      wbuf[15]=(cp->vme_bto)&0xFF;
      wbuf[16]=((cp->vme_bgto)>>8)&0xFF;
      wbuf[17]=(cp->vme_bgto)&0xFF;
      nwbuf+=14;
      break;
    default:
      printf("illeagal CR ID\n");
      err=-1;
      break;
  }
  if(err!=0){
    return(err);
  }
  n=eth_write();
  for(l=0;l<8000;l++)lcnt++;
  return(n);
}

/*
// wrt_crs the way it is implemented now.
int wrt_crs(enum CR_ID crid, CNFG_ptr cp)
{
  int i,n,l,lcnt,err;
  err=0;
  wbuf[0]=0x00;
  nwbuf=2;
  switch(crid){
    case ETHER:
      wbuf[1]=Wrt_Eth_CR;
      wbuf[2]=((cp->ether)>>8)&0xFF;
      wbuf[3]=(cp->ether)&0xFF;
      nwbuf+=2;
      break;
    case EXTFIFO:
      wbuf[1]=Wrt_Ext_CR;
      wbuf[2]=((cp->ext_fifo)>>8)&0xFF;
      wbuf[3]=(cp->ext_fifo)&0xFF;
      nwbuf+=2;
      break;
    case RESET:
      wbuf[1]=Wrt_Rst_CR;
      wbuf[2]=((cp->reset)>>8)&0xFF;
      wbuf[3]=(cp->reset)&0xFF;
      nwbuf+=2;
      break;
    case VME:
      wbuf[1]=Wrt_VME_CR;
      wbuf[2]=((cp->vme)>>24)&0xFF;
      wbuf[3]=((cp->vme)>>16)&0xFF;
      wbuf[4]=((cp->vme)>>8)&0xFF;
      wbuf[5]=(cp->vme)&0xFF;
      nwbuf+=4;
      break;
    case BTO:
      wbuf[1]=Wrt_BTO_CR;
      wbuf[2]=((cp->vme_bto)>>8)&0xFF;
      wbuf[3]=(cp->vme_bto)&0xFF;
      nwbuf+=2;
      break;
    case BGTO:
      wbuf[1]=Wrt_BGTO_CR;
      wbuf[2]=((cp->vme_bgto)>>8)&0xFF;
      wbuf[3]=(cp->vme_bgto)&0xFF;
      nwbuf+=2;
      break;
    case ALL_CRS:
      wbuf[1]=Wrt_All_CRs;
      wbuf[2]=((cp->ether)>>8)&0xFF;
      wbuf[3]=(cp->ether)&0xFF;
      wbuf[4]=((cp->ext_fifo)>>8)&0xFF;
      wbuf[5]=(cp->ext_fifo)&0xFF;
      wbuf[6]=((cp->reset)>>8)&0xFF;
      wbuf[7]=(cp->reset)&0xFF;
      wbuf[8]=((cp->vme)>>24)&0xFF;
      wbuf[9]=((cp->vme)>>16)&0xFF;
      wbuf[10]=((cp->vme)>>8)&0xFF;
      wbuf[11]=(cp->vme)&0xFF;
      wbuf[12]=((cp->vme_bto)>>8)&0xFF;
      wbuf[13]=(cp->vme_bto)&0xFF;
      wbuf[14]=((cp->vme_bgto)>>8)&0xFF;
      wbuf[15]=(cp->vme_bgto)&0xFF;
      nwbuf+=14;
      break;
    default:
      printf("illeagal CR ID\n");
      err=-1;
      break;
  }
  if(err!=0){
    return(err);
  }
  n=eth_write();
  for(l=0;l<8000;l++)lcnt++;
  return(n);
}
*/

int set_clr_bits(enum SET_CLR sc, enum CR_ID crid, unsigned int mask)
{
  int i,n,l,lcnt,err;
  err=0;
  wbuf[0]=0x00;
  wbuf[1]=Set_Clr_CRs;
  wbuf[2]=sc;
  wbuf[3]=crid;
  nwbuf=4;
  switch(crid){
    case ETHER: case EXTFIFO: case RESET: case BTO: case BGTO:
      wbuf[4]=(mask>>8)&0xFF;
      wbuf[5]=mask&0xFF;
      nwbuf+=2;
      break;
    case VME:
      wbuf[4]=(mask>>24)&0xFF;
      wbuf[5]=(mask>>16)&0xFF;
      wbuf[6]=(mask>>8)&0xFF;
      wbuf[7]=(mask)&0xFF;
      nwbuf+=4;
      break;
    default:
      printf("illeagal CR ID\n");
      err=-1;
      break;
  }
  if(err!=0){
    return(err);
  }
  n=eth_write();
  for(l=0;l<8000;l++)lcnt++;
  return(n);
}

int wrt_ser_num(int ser_num)
{
  int n,l,lcnt;
  wbuf[0]=0x00;
  wbuf[1]=Wrt_Ser_Num;
  wbuf[2]=0x00;
  wbuf[3]=(ser_num>>16)&0xFF;
  wbuf[4]=(ser_num>>8)&0xFF;
  wbuf[5]=ser_num&0xFF;
  nwbuf=6;
  n=eth_write();
  for(l=0;l<8000;l++)lcnt++;
  return(n);
}

int rd_ser_num()
{
  int n,type;
  wbuf[0]=0x20;
  wbuf[1]=Rd_Ser_Num;
  nwbuf=2;
  n=eth_write();
  n=eth_read();
  type = rbuf[15]&0xff;
  if(type==0xFF){
    return(-(((rbuf[22]&0xFF)<<8)|(rbuf[23]&0xFF)));
  }
  return(((rbuf[23]&0xFF)<<16)|((rbuf[24]&0xFF)<<8)|(rbuf[25]&0xFF));
}

void cp_macs(CNFG_ptr to, CNFG_ptr from)
{
  int i;
  for(i=0;i<6;i++)to->mac.device[i]=from->mac.device[i];
  for(i=0;i<6;i++)to->mac.mcast1[i]=from->mac.mcast1[i];
  for(i=0;i<6;i++)to->mac.mcast2[i]=from->mac.mcast2[i];
  for(i=0;i<6;i++)to->mac.mcast3[i]=from->mac.mcast3[i];
  for(i=0;i<6;i++)to->mac.dflt_srv[i]=from->mac.dflt_srv[i];
  return;
}
void cp_crs(CNFG_ptr to, CNFG_ptr from)
{
  int i;
  to->ether    = from->ether;
  to->ext_fifo = from->ext_fifo;
  to->reset    = from->reset;
  to->vme      = from->vme;
  to->vme_bto  = from->vme_bto;
  to->vme_bgto = from->vme_bgto;
  return;
}

void cp_cnfg(CNFG_ptr to, CNFG_ptr from)
{
  cp_macs(to, from);
  cp_crs(to, from);
  return;
}

void print_macs(CNFG_ptr cp)
{
  int i;
  printf("              MAC address\n---------------------------\n");
  printf("Device:   ");
  for(i=0;i<5;i++)printf("%02X-",cp->mac.device[i]);
  printf("%02X\n",cp->mac.device[5]);
  printf("Mcast1:   ");
  for(i=0;i<5;i++)printf("%02X-",cp->mac.mcast1[i]);
  printf("%02X\n",cp->mac.mcast1[5]);
  printf("Mcast2:   ");
  for(i=0;i<5;i++)printf("%02X-",cp->mac.mcast2[i]);
  printf("%02X\n",cp->mac.mcast2[5]);
  printf("Mcast3:   ");
  for(i=0;i<5;i++)printf("%02X-",cp->mac.mcast3[i]);
  printf("%02X\n",cp->mac.mcast3[5]);
  printf("Dflt_Srv: ");
  for(i=0;i<5;i++)printf("%02X-",cp->mac.dflt_srv[i]);
  printf("%02X\n\n",cp->mac.dflt_srv[5]);
  return;
}
void print_crs(CNFG_ptr cp)
{
  printf("           Config Registers\n---------------------------\n");
  printf("Ether:       %04hX\n",cp->ether);
  printf("Ext_FIFO:    %04hX\n",cp->ext_fifo);
  printf("Reset:       %04hX\n",cp->reset);
  printf("VME:     %08X\n",cp->vme);
  printf("VME_BTO:     %04hX\n",cp->vme_bto);
  printf("VME_BGTO:    %04hX\n\n",cp->vme_bgto);
  return;
}
void print_cnfg(CNFG_ptr cp)
{
  print_macs(cp);
  print_crs(cp);
  return;
}
